// Java-Applet Fadenpendel (Beispiel fr eine einfache Animation)
// Kompilierung mit J2SDK 1.6.0
// Walter Fendt, 
// Vorschlge zur didaktischen Reduzierung
// Claus Jrgen Schmitt
// 04.04.2009

import java.awt.*;
import javax.swing.*;

public class Fadenpendel extends JApplet implements Runnable {

//-----------------------------------------------------------------------------	
// Konstanten
//-----------------------------------------------------------------------------
  
  final int R = 20;                         // Radius Pendelkrper (Pixel)
  final int R2 = 2*R;                       // Durchmesser Pendelkrper (Pixel)
  final int L = 200;                        // Pendellnge (Pixel)
  final double A = 0.4;                     // Amplitude (Bogenma); entspricht etwa 23
  final double T = 2;                       // Schwingungsdauer (s)
  final double OMEGA = 2*Math.PI/T;         // Kreisfrequenz (1/s)  
  final Color FZF = Color.yellow;           // Hintergrundfarbe Zeichenflche
   
//----------------------------------------------------------------------------- 
// Attribute
//-----------------------------------------------------------------------------
  
  Container cp;                             // Container fr Komponenten
  int breite, hhe;                         // Abmessungen (Pixel)
  Zeichenflche zf;                         // Zeichenflche
  int ax, ay;                               // Aufhngung des Pendels (Pixel)
  double phi;                               // Phase (Bogenma)
  Thread thr;                               // Prozess fr Animation
  double t;                                 // Zeit (s)
  
//-----------------------------------------------------------------------------
// Weitere Methoden
//-----------------------------------------------------------------------------

  // Start-Methode:

  public void start () {
    cp = getContentPane();                  // Container fr Komponenten
    cp.setLayout(null);                     // Hartes Layout (Pixelangaben)
    Dimension dim = getSize();              // Abmessungen (aus HTML-Datei)
    breite = dim.width;                     // Breite (Pixel) 
    hhe = dim.height;                      // Hhe (Pixel)
    ax = breite/2;                          // x-Koordinate Aufhngung (Pixel)
    ay = 40;                                // y-Koordinate Aufhngung (Pixel) 
    zf = new Zeichenflche();               // Zeichenflche (innere Klasse) 
    zf.setBackground(FZF);                  // Hintergrundfarbe Zeichenflche
    zf.setBounds(0,0,breite,hhe);          // Position/Abmessungen Zeichenflche
    cp.add(zf);                             // Zeichenflche hinzufgen                       
    thr = new Thread(this);                 // Prozess fr Animation 
    thr.start();                            // Prozess starten
    }

  // Stopp-Methode:

  public void stop () {
    thr = null;                             // Prozess fr Garbage Collection freigeben 
    cp.removeAll();                         // Komponenten entfernen
    }

  // Methode fr Animation (Schleife ber eine Schwingung):

  public void run () {
    long t0, t1;
    t0 = System.currentTimeMillis();        // Zeit in ms
    while (thr == Thread.currentThread()) { // Solange Prozess thr aktiv ...
      if (t <= T)                           // ... Falls erste Periode ... 
        zf.repaint();                       // ... Zeichenflche neu zeichnen
      try {Thread.sleep(50);}               // ... 50 ms Unterbrechung 
      catch (InterruptedException ie) {}
      t1 = System.currentTimeMillis();      // ... Zeit in ms
      t += (t1-t0)/1000.0;                  // ... Zeit-Variable aktualisieren (in s)
      t0 = t1;                              // ... Zeit in ms
      }
    }

  // Pendel zeichnen:
  // g ... Grafik-Kontext

  void pendel (Graphics g) {
    // Zur Zeit t = 0 ist das Pendel_1 nach links maximal ausgelenkt.  
    phi = A*Math.cos(OMEGA*t);				// Phase (Bogenma)
	  									    // Im Gradma Auslenkung etwa von 23 bis -23
    int px, py;                             // Mittelpunktskoordinaten Pendelkrper 
    px = (int)Math.round(ax-L*Math.sin(phi));  // x-Koordinate Kugelmittelpunkt
    py = (int)Math.round(ay+L*Math.cos(phi));  // y-Koordinate Kugelmittelpunkt     
    g.setColor(Color.black);                // Faden 
    g.drawLine(ax,ay,px,py); 
    g.setColor(Color.blue);                 // Pendelkrper (Kreisflche) 
    g.fillOval(px-R,py-R,R2,R2); 
    g.setColor(Color.black);                // Pendelkrper (Kreisrand) 
    g.drawOval(px-R,py-R,R2,R2);
    }
    
  // --------------------------------------------------------------------------
  
  // Innere Klasse Zeichenflche:
  
  class Zeichenflche extends JPanel {
  	
    // Grafik-Ausgabe:
    // g ... Grafik-Kontext
  	
    public void paint (Graphics g) {
      super.paint(g);                       // Hintergrund zeichnen
      g.setColor(Color.black);              // Aufhngung zeichnen 
      g.fillRect(40,ay-4,breite-80,4);
      pendel(g); 							// Pendel zeichnen
      }
      
    } // Ende Zeichenflche

  } // Ende Fadenpendel






