// Java-Applet Pythagoras 
// (Kathetensatz und Satz des Pythagoras)
// Kompilierung mit J2SDK 1.6.0
// Walter Fendt
// 25.10.1997 - 30.03.2009
// Reduzierung auf den Satz des Thales:
// Experimentelles Entdecken (Grnfrben des Dreiecks bei rechtem Winkel)
// Grndonnerstag - 20.04.2009
// C.Schmitt

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Thales extends JApplet implements MouseMotionListener {
	
//-----------------------------------------------------------------------------
// Konstanten
//-----------------------------------------------------------------------------

  final Color FZF = Color.yellow;           // Hintergrundfarbe Zeichenflche
  final Font SANSSERIF                      // Logischer Font
    = new Font("SansSerif",Font.BOLD,12);
  
//-----------------------------------------------------------------------------
// Attribute
//-----------------------------------------------------------------------------

  Zeichenflche zf;                         // Zeichenflche
  int breite, hhe;                         // Abmessungen (Pixel)
  int xm, ym;           	   			    // Mittelpunkt des Thaleskreises
  int r;                        	        // Radius 
  int r2;								    // Durchmesser
  int xa, xb ;     				  		    // x-Koordinaten der Hypotenusen-Endpunkte
  Polygon dreieck;                          // Dreieck
  Color dreiecksFarbe;						// jeweilige Dreiecksfarbe (rot / grn spter)
  
//-----------------------------------------------------------------------------
// Methoden
//-----------------------------------------------------------------------------

// Start-Methode:

  public void start () {
  	Container cp = getContentPane();        // Container fr Komponenten
  	cp.setLayout(null);                     // Hartes Layout (Pixelangaben)
  	Dimension dim = getSize();              // Abmessungen (aus HTML-Datei)
  	breite = dim.width;                     // Breite (Pixel) 
  	hhe = dim.height;                      // Gesamthhe (Pixel)
  	xm = breite / 2;  					    // Koordinaten fr   
  	ym = hhe * 8 / 10;     				// Thaleskreis-Mittelpunkt
  	r = breite / 2 - 10;				    // Radius
  	r2 = 2*r;								// Durchmesser
  	xa = xm - r; xb = xm + r;       	    // End-x-Koordinaten der Hypotenuse
    zf = new Zeichenflche();               // Zeichenflche
    zf.setBackground(FZF);                  // Hintergrundfarbe Zeichenflche
    zf.setBounds(0,0,breite,hhe);          // Position/Abmessungen Zeichenflche
    cp.add(zf); 				            // Zeichenflche hinzufgen 
    dreieck = neuesDreieck();               // Dreieck
    dreiecksFarbe = Color.green;			// Grne Farbe fr das rechtwinklige Start-Dreieck
    setzenPunkt(0,xa,ym);         			// Endpunkte der Hypotenuse setzen (konstant)
    setzenPunkt(1,xb,ym);
    setzenPunkt(2,xm,ym-r);					// Startscheitelpunkt setzen
    										// (geichschenkliges Thalesdreieck)
    zf.repaint(); 							// Zeichenflche neu zeichnen
    zf.addMouseMotionListener(this);        // Aufpasser fr Mausbewegungen
    }
    
  // Erzeugung eines Dreiecks:
  
  
  Polygon neuesDreieck () {
  	Polygon p = new Polygon();              // Neues Polygon (mit 0 Ecken)
  	for (int i=0; i<3; i++)                 // 3 Ecken hinzufgen                  
  	  p.addPoint(0,0);
  	return p;                               // Rckgabewert (Referenz auf Polygon) 
    }

  // Setzen eines Punktes (Dreiecksecke):
  // i ...... Index der Ecke
  // x, y ... Koordinaten

  void setzenPunkt (int i, int x, int y) {
    dreieck.xpoints[i] = x; 
    dreieck.ypoints[i] = y;
    } 
   
  // Reaktion auf Ziehen der Maus:
  
  public void mouseMoved (MouseEvent me) {
    int x = me.getX(), y = me.getY();       // Position des Mauszeigers                   
    if (y > ym) y = ym;                     // Mauszeiger unterhalb der Hypotenuse
    double dx = xm-x, dy = ym-y;            // Koordinaten-Differenzen 
    										// Ma fr die Abweichung vom rechtwinkligen Dreieck
    double diff = Math.abs(Math.sqrt(dx*dx+dy*dy)-r);
    										// Dreiecksfarbe festlegen
    if (diff < 0.5)
      dreiecksFarbe = Color.green;
    else
      dreiecksFarbe = Color.red;    	
    setzenPunkt(2,x,y);				        // Scheitelpunkt setzen fr Dreieck
    zf.repaint(); 							// Neu zeichnen
    }
                                   
    
    
  // Nicht bentigte Methoden von MouseMotionListener:
  public void mouseDragged (MouseEvent me){}  
  public void mouseEntered (MouseEvent me){}
  public void mouseExited (MouseEvent me) {}
    
  // ----------------------------------------------------------------

  // Innere Klasse Zeichenflche:
  
  class Zeichenflche extends JPanel {
  	public void paint (Graphics g) {
  	  super.paint(g);                       	// Hintergrund zeichnen
  	  
      g.setColor(dreiecksFarbe);     			// Dreieck ausfllen 
      g.fillPolygon(dreieck);        
      g.setColor(Color.black);					// Rnder zeichnen  
      g.drawArc(xm-r,ym-r,r2,r2,0,180);       	// Thaleskreis zeichnen
      											// Halbkreise fr zwei weitere Radien zeichnen,
      											// um Thaleskreis besser zu betonen
      int rr, d;							    // Hilfsgren dafr
      rr = r - 1;								
      d = 2*rr;
      g.drawArc(xm-rr,ym-rr,d,d,0,180);
      rr = r + 1;
      d = 2*rr;
      g.drawArc(xm-rr,ym-rr,d,d,0,180);
      											// Dreieckseiten zeichnen
      g.drawPolygon(dreieck);
      g.setFont(SANSSERIF);                		// Header
      g.drawString("Dreieck mit der Maus erfahren",breite/3,40);
      											// Copyright-Hinweis
      g.drawString("  W. Fendt 1997",50,hhe-20); 
      g.drawString("reduziert  von  C. Schmitt 2009",50,hhe-7);
      }
      
    } // Ende Zeichenflche

  } // Ende Thales


